@@ -15,7 +15,7 @@ using a Metropolis-Hastings Monte Carlo algorithm
  works and are not responsible for any damage you may cause or have.
  
  
- $Id: bayes.c,v 1.9 2002/07/12 18:14:08 beerli Exp $
+ $Id: bayes.c,v 1.10 2002/08/12 21:13:51 beerli Exp $
 */
 
 #include "bayes.h"
@@ -159,8 +159,8 @@ void bayes_save(world_fmt *world)
     memcpy(&world->bayes->params[pnum][1], world->param0,sizeof(double)*world->numpop2);
     //DEBUG
     //for(i=0; i<world->numpop2+1;i++)
-    //    fprintf(stderr,"%f ",world->bayes->params[pnum][i]);
-    //fprintf(stderr,"\n");
+    //    FPRINTF(stderr,"%f ",world->bayes->params[pnum][i]);
+    //FPRINTF(stderr,"\n");
 
     pnum++;
     if(pnum>=allocparams)
@@ -256,29 +256,29 @@ void bayes_stat(world_fmt *world)
 	  }
 	for(i=0; i< bayes->numparams; i++)
     {
-		fprintf(world->bayesfile, "%f ", bayes->params[i][0]);
+		FPRINTF(world->bayesfile, "%f ", bayes->params[i][0]);
 		weight =  exp(bayes->params[i][0] -  bayes->params[mode][0]);
         for(j = 0; j < size; j++)
         {
 			   temp = bayes->params[i][j+1];
-            fprintf(world->bayesfile, "%f ", temp);
+            FPRINTF(world->bayesfile, "%f ", temp);
             waverages[j] += weight*temp;
             wstd[j] += weight * temp * temp;
             averages[j] += temp;
             std[j] += temp * temp;
             rawstat[j][i] = temp;
         }
-        fprintf(world->bayesfile, "\n");
+        FPRINTF(world->bayesfile, "\n");
     }
-	fprintf(world->outfile,"\n\n\nBayesian estimates\n");
-	fprintf(world->outfile,"==================\n\n");
-	fprintf(world->outfile,"Parameter 2.5%%  median 97.5%%  mode   mean  std  wmean   wstd\n");
-	fprintf(world->outfile,"------------------------------------------------------------------------------\n");
+	FPRINTF(world->outfile,"\n\n\nBayesian estimates\n");
+	FPRINTF(world->outfile,"==================\n\n");
+	FPRINTF(world->outfile,"Parameter 2.5%%  median 97.5%%  mode   mean  std  wmean   wstd\n");
+	FPRINTF(world->outfile,"------------------------------------------------------------------------------\n");
     for(j=0; j< size; j++)
     {
         qsort(rawstat[j],bayes->numparams,sizeof(double),numcmp);
-        fprintf(world->outfile,"  %3li   ",j+1);
-		fprintf(world->outfile, " %.5f %.5f %.5f %.5f %.5f %.5f %.5f %.5f\n",
+        FPRINTF(world->outfile,"  %3li   ",j+1);
+		FPRINTF(world->outfile, " %.5f %.5f %.5f %.5f %.5f %.5f %.5f %.5f\n",
 		  rawstat[j][(long)(bayes->numparams * 0.025)],
 		  rawstat[j][(long)(bayes->numparams * 0.5)],
 		  rawstat[j][(long)(bayes->numparams * 0.975)],
