@@ -4,7 +4,7 @@
 //
 // started November 2000
 // Peter Beerli Seattle 2000
-// $Id: bayes.h,v 1.5 2002/06/20 06:25:33 beerli Exp $
+// $Id: bayes.h,v 1.6 2002/07/05 22:14:24 beerli Exp $
 //
 #include "migration.h"
 #ifdef BAYESUPDATE
@@ -17,7 +17,7 @@ extern void bayes_free(world_fmt *world);
 extern void bayes_fill(world_fmt *world, option_fmt *options);
 extern void bayes_init(bayes_fmt *bayes, long size);
 extern void bayes_save(world_fmt *world);
-extern void bayes_stat(world_fmt *world, long size);
+extern void bayes_stat(world_fmt *world);
 
 #endif /*baysupdate*/
 
