@@ -18,7 +18,7 @@ using a Metropolis-Hastings Monte Carlo algorithm
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
- $Id: broyden.c,v 1.64 2002/06/20 06:25:33 beerli Exp $
+ $Id: broyden.c,v 1.65 2002/08/12 21:13:52 beerli Exp $
 -------------------------------------------------------*/
 extern double norm (double *d, long size);
 
@@ -1303,7 +1303,7 @@ gradient_longsum (double *d, nr_fmt * nr, long locus)
     long ptime, ptimeold;
     double timeinterval;
     //interval[0]=intervals[1]=intervals[2]=0;
-    fprintf(stdout,"rawrates=%f %f %f @ %f %f %f \n",rawrates[0],rawrates[1],rawrates[2],rawrates[3],rawrates[4],rawrates[5]);
+    FPRINTF(stdout,"rawrates=%f %f %f @ %f %f %f \n",rawrates[0],rawrates[1],rawrates[2],rawrates[3],rawrates[4],rawrates[5]);
     treeparts = calloc(nr->numpop2+numpop3,sizeof(double));
     memset (d, 0, sizeof (double) * nr->partsize);
     thetas = nr->param;
@@ -1396,9 +1396,9 @@ gradient_longsum (double *d, nr_fmt * nr, long locus)
             }
             //   for (pop = 0; pop < 3; pop++)
             //    {
-            //  fprintf(stdout,"%li ", intervals[pop]);
+            //  FPRINTF(stdout,"%li ", intervals[pop]);
             //  }
-            // fprintf(stdout,"\n");
+            // FPRINTF(stdout,"\n");
 
             z = 0;
             for (i = 0; i < nn; i++)
@@ -1868,17 +1868,17 @@ probG4_longsum (double *fullparam, longsum_fmt *data, long longsumlen, long nump
                     result += timeinterval * (tint->lineages2[i] / rawrates[ptimeold] * fullparam[i+numpop2] + tint->lineages[i] * sm[i]);
                     timeinterval = time_end - rawtimes[ptime];
                     result +=  timeinterval * (tint->lineages2[i] / rawrates[ptime] * fullparam[i+numpop2] + tint->lineages[i] * sm[i]);
-                    //     fprintf(stderr,"on border timeend %f, interval %f, rawrate %f\n",time_end,tint->interval, rawrates[ptime]);
+                    //     FPRINTF(stderr,"on border timeend %f, interval %f, rawrate %f\n",time_end,tint->interval, rawrates[ptime]);
                 }
             }
             else
             {
                 result +=  timeinterval * (tint->lineages2[i] / rawrates[ptime] * fullparam[i+numpop2] + tint->lineages[i] * sm[i]);
             }
-            //   fprintf(stdout,"p=%i t=%f, u=%f ptime=%li rt=%f res=%f\n", i, time_end, timeinterval, ptime, rawtimes[ptime], result);
+            //   FPRINTF(stdout,"p=%i t=%f, u=%f ptime=%li rt=%f res=%f\n", i, time_end, timeinterval, ptime, rawtimes[ptime], result);
         }
     }
-    // fprintf(stdout,"\n");
+    // FPRINTF(stdout,"\n");
     //free(rates);
     //  free(rtimes);
     return result;
@@ -2078,7 +2078,7 @@ print_contribution (nr_fmt * nr, timearchive_fmt ** atl, long G)
     if (nr->world->options->mixplot)
     {
         mixfile = fopen ("mixfile", "a");
-        fprintf (mixfile, "\n\n");
+        FPRINTF (mixfile, "\n\n");
     }
     maxtemp = -DBL_MAX;
     for (r = nr->repstart; r < nr->repstop; r++)
@@ -2096,7 +2096,7 @@ print_contribution (nr_fmt * nr, timearchive_fmt ** atl, long G)
                 for (i = nr->world->numpop; i < nr->world->numpop2; i++)
                     events += (long) tyme[g].mindex[i];
                 for (i = 0; i < tyme[g].copies; i++)
-                    fprintf (mixfile, "%li %f %f ", events, temp,
+                    FPRINTF (mixfile, "%li %f %f ", events, temp,
                              nr->world->likelihood[g]);
             }
             temp2 -= maxtemp;
@@ -2122,19 +2122,19 @@ void
 print_menu_contribution (FILE * file, long contribution[])
 {
     long g;
-    fprintf (file, "           log(P(g|Param) * P(data|g)\n");
-    fprintf (file, "                            -20 to ");
+    FPRINTF (file, "           log(P(g|Param) * P(data|g)\n");
+    FPRINTF (file, "                            -20 to ");
     for (g = -18; g <= 0; g += 2)
     {
-        fprintf (file, "%4li ", g);
+        FPRINTF (file, "%4li ", g);
     }
-    fprintf (file, "  All\n");
-    fprintf (file, "           Counts                  ");
+    FPRINTF (file, "  All\n");
+    FPRINTF (file, "           Counts                  ");
     for (g = 0; g < 10; g++)
     {
-        fprintf (file, "%4li ", contribution[g]);
+        FPRINTF (file, "%4li ", contribution[g]);
     }
-    fprintf (file, "%5li\n", contribution[10]);
+    FPRINTF (file, "%5li\n", contribution[10]);
 }
 
 /* calculates log(parameters)*/
@@ -2172,7 +2172,7 @@ create_apg0 (double *apg0, nr_fmt * nr, timearchive_fmt * tyme, long locus)
     long g;
     long copies;
     /* Prob(G|Param0) */
-    //  fprintf(stdout,"first 4 theta0: %f %f %f %f \n",tyme->param0[0], tyme->param0[1], tyme->param0[2], tyme->param0[3]);
+    //  FPRINTF(stdout,"first 4 theta0: %f %f %f %f \n",tyme->param0[0], tyme->param0[1], tyme->param0[2], tyme->param0[3]);
     for (g = 0; g < tyme->T; g++)
     {
         if (g > 0)
@@ -2192,7 +2192,7 @@ create_apg0 (double *apg0, nr_fmt * nr, timearchive_fmt * tyme, long locus)
 
         }
     }
-    //   fprintf(stdout,"first 4 apg0: %f %f %f %f \n",apg0[0], apg0[1], apg0[2], apg0[3]);
+    //   FPRINTF(stdout,"first 4 apg0: %f %f %f %f \n",apg0[0], apg0[1], apg0[2], apg0[3]);
 
 }
 
