@@ -6,7 +6,7 @@
  M A X I M I Z E R      R O U T I N E S
  
  - calculates single locus ML
- - mulitlocus ML with constant mutation rate
+ - multilocus ML with constant mutation rate
  - multilocus ML with gamma deviated mutation rate 
  
   using Broyden-Fletcher-Goldfarb-Shanno method
@@ -23,7 +23,7 @@
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: combroyden2.c,v 1.81 2002/07/18 14:29:44 beerli Exp $
+$Id: combroyden2.c,v 1.82 2002/08/12 21:13:52 beerli Exp $
  
 -------------------------------------------------------*/
 
@@ -236,7 +236,7 @@ estimateParameter (long rep, long G, world_fmt * world, option_fmt * options,
         if (kind == SINGLELOCUS
                 && world->options->custm2[world->numpop2] == 'c')
             trials = maximize (param, world, nr, kind, repkind);
-        else
+        else 
             trials = do_profiles (world, nr, &likes, &normd, kind, rep, repkind);
     }
     else
@@ -246,16 +246,12 @@ estimateParameter (long rep, long G, world_fmt * world, option_fmt * options,
     {
         print_menu_chain (type, chain, world->atl[rep][world->locus].T,
                           world, options, rep);
-        fprintf (stdout, "           Maximization cycles needed: %li\n",
-                 trials);
-        fprintf (stdout, "           Norm of first derivatives:  %f\n",
-                 nr->normd);
+        FPRINTF (stdout, "           Maximization cycles needed: %li\n           Norm of first derivatives:  %f\n",
+                 trials, nr->normd);
         if (world->options->logfile)
         {
-            fprintf (world->options->logfile,
-                     "           Maximization cycles needed: %li\n", trials);
-            fprintf (world->options->logfile,
-                     "           Norm of first derivatives:  %f\n", nr->normd);
+            FPRINTF (world->options->logfile,
+                     "           @Maximization cycles needed: %li\n           Norm of first derivatives:  %f\n", trials, nr->normd);
         }
     }
     if (kind == MULTILOCUS && world->options->plotnow && world->loci > 1)
@@ -574,10 +570,10 @@ print_menu_finalestimate (boolean progress, long locus, long loci)
     {
         get_time (nowstr, "%H:%M:%S");
         if (locus < loci)
-            fprintf (stdout, "%s   Parameter estimation for locus %li\n", nowstr,
+            FPRINTF (stdout, "%s   Parameter estimation for locus %li\n", nowstr,
                      locus);
         else
-            fprintf (stdout, "%s   Parameter estimation over all loci\n", nowstr);
+            FPRINTF (stdout, "%s   Parameter estimation over all loci\n", nowstr);
     }
 }
 
@@ -595,10 +591,10 @@ print_reset_big_param (FILE * file, double average, double maxtheta, long pop)
     if (!checkparam[pop])
     {
         get_time (nowstr, "%H:%M:%S");
-        fprintf (file,
+        FPRINTF (file,
                  "%s   Theta for population %li was reset to the average\n",
                  nowstr, pop);
-        fprintf (file, "          of %f, it exceeded the maximum of %f \n",
+        FPRINTF (file, "          of %f, it exceeded the maximum of %f \n",
                  average, maxtheta);
     }
 }
@@ -849,6 +845,8 @@ line_search (double *dv, double *gxv,
     double psiabc;
     double ll, la, lb, lc;
     double m = 1.0;
+    double denom;
+    double nom;
     helper_fmt helper;
     helper = *thishelper;
     //  partcopy_helper(thishelper, &helper);
@@ -867,14 +865,17 @@ line_search (double *dv, double *gxv,
     while (trials++ < NTRIALS)
     {
         //            printf ("*%3li> %f %f %f / %f %f %f\n", trials, a, b, c, la, lb, lc);
-        locallamda =
-            (c * c * (lb - la) + b * b * (la - lc) +
-             a * a * (lc - lb)) / (2. * (b * la - c * la - a * lb + c * lb +
-                                         a * lc - b * lc));
-        if(isnan(locallamda))
-            return 1.;
-        psiabc = ((la - lb) / (-a + b) + (la - lc) / (a - c)) / (-b + c);
-        if ((psiabc <= 0.0) || (locallamda >= m))
+      denom = (2. * (b * la - c * la - a * lb + c * lb +  a * lc - b * lc));
+      if((fabs(denom)-DBL_EPSILON)>DBL_EPSILON)
+	{
+	  nom = (c * c * (lb - la) + b * b * (la - lc) +
+		 a * a * (lc - lb));
+	}
+      else
+	return 1.; 
+      locallamda = nom / denom;
+      psiabc = ((la - lb) / (-a + b) + (la - lc) / (a - c)) / (-b + c);
+      if ((psiabc <= 0.0) || (locallamda >= m))
         {
             if ((a == m || b == m || c == m))
                 return m;
@@ -1268,18 +1269,18 @@ report_broyden (double newL, double normd, long trials,
                 FILE * logfile)
 {
     if (boolgamma)
-        fprintf (stdout, "%i:%li> Log(L)=%f Norm=%f Alpha=%f Theta_1=%f\n",
+        FPRINTF (stdout, "%i:%li> Log(L)=%f Norm=%f Alpha=%f Theta_1=%f\n",
                  myID, trials, newL, normd, alpha, theta1);
     else
-        fprintf (stdout, "%i:%li> Log(L)=%f Norm=%f Theta_1=%f\n",
+        FPRINTF (stdout, "%i:%li> Log(L)=%f Norm=%f Theta_1=%f\n",
                  myID, trials, newL, normd, theta1);
     if (logfile != NULL)
     {
         if (boolgamma)
-            fprintf (logfile, "%i:%li> Log(L)=%f Norm=%f Alpha=%f Theta_1=%f\n",
+            FPRINTF (logfile, "%i:%li> Log(L)=%f Norm=%f Alpha=%f Theta_1=%f\n",
                      myID, trials, newL, normd, alpha, theta1);
         else
-            fprintf (logfile, "%i:%li> Log(L)=%f Norm=%f Theta_1=%f\n",
+            FPRINTF (logfile, "%i:%li> Log(L)=%f Norm=%f Theta_1=%f\n",
                      myID, trials, newL, normd, theta1);
     }
 }
@@ -1405,7 +1406,7 @@ maximize (double *param, world_fmt * world, nr_fmt * nr, long analystype,
     set_expparam (expxv, xv, nn);
     //SETUPPARAM0 (world, nr, world->repkind, repstart, repstop,
     //             world->loci, analystype, helper.multilocus);
-    //   fprintf(stdout,"analystype is %li\n",analystype);
+    //   FPRINTF(stdout,"analystype is %li\n",analystype);
 
     fill_helper (&helper, expxv, xv, world, nr);
     oldL = CALCLIKE (&helper, expxv, xv);
@@ -1521,7 +1522,7 @@ finish_broyden (double newL, double normd, long trials,
     //correct_values (world, nr); // resets huge thetas to average
 
     if (world->options->verbose)
-        fprintf (stdout, "\n");
+        FPRINTF (stdout, "\n");
     switch (analystype)
     {
     case MULTILOCUS:
@@ -1652,3 +1653,4 @@ fill_helper (helper_fmt * helper, double *param, double *lparam,
     helper->weight = denom;
     helper->atl = world->atl;
 }
+
