@@ -572,7 +572,7 @@ read_microalleles (FILE * infile, data_fmt * data, long pop, long ind)
         if (input[0] == '\0')
             FGETS (input, LONGLINESIZE, infile);
         i = 0;
-        while (strchr(" \t",input[i]) && input[i] != dlm[0])
+        while (strchr(" \t",input[i])==NULL && input[i] != dlm[0])
         {
             a1[i] = input[i];
             i++;
@@ -583,7 +583,7 @@ read_microalleles (FILE * infile, data_fmt * data, long pop, long ind)
         if (input[i] == dlm[0])
         {
             input++;
-            while (strchr(" \t",input[i]) && input[i] != '\0')
+            while (strchr(" \t",input[i])==NULL && input[i] != '\0')
             {
                 a2[i] = input[i];
                 i++;
