@@ -21,7 +21,7 @@ and of course the documentation at:
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: main.c,v 1.83 2002/07/12 18:14:08 beerli Exp $
+$Id: main.c,v 1.85 2002/08/12 21:13:53 beerli Exp $
 -------------------------------------------------------*/
 #include "migration.h"
 #include "definitions.h"
@@ -64,7 +64,12 @@ int             myID;  /* used if MPI is running otherwise == 0       */
 int             color;  /* either 1 or MPI_UNDEFINED */
 int             numcpu;  /* used if MPI is running otherwise == 1     */
 int             locidone; /* used in MPI workers */
+
+
 #ifdef MPI
+filedb_fmt filedb[20];
+long filenum;
+
 MPI_Comm        comm_world;
 #endif
 #ifdef SLOWNET
@@ -211,6 +216,7 @@ main(int argc, char **argv)
 #endif
 #ifdef MPI
     // parallel version
+	filenum = 0; //setting up the filename database so tht worker node can write back to master
     MPI_Init(&argc, &argv);
     comm_world = MPI_COMM_WORLD;
     MPI_Comm_size(comm_world, &numcpu);
@@ -268,7 +274,7 @@ main(int argc, char **argv)
     else
     {
         broadcast_options_worker(options);
-        options->writelog = FALSE;
+        //options->writelog = FALSE;
         options->menu = FALSE;
     }
 #endif
@@ -281,7 +287,7 @@ main(int argc, char **argv)
     {
         init_files(EARTH, data, options);
     }
-    if (options->writelog)
+    if (options->writelog && myID==MASTER)
         print_menu_title(options->logfile, options);
     EARTH->repkind = SINGLECHAIN;
     unset_penalizer_function(FALSE); // install penalizer for far jumps in MCMC
@@ -571,7 +577,7 @@ run_sampler(option_fmt * options, data_fmt * data, world_fmt ** universe,
 
     if (myID == MASTER)
     {
-        mpi_runloci_master(data->loci, EARTH->who);
+        mpi_runloci_master(data->loci, EARTH->who,EARTH);
     }
     else
     {
@@ -830,8 +836,8 @@ run_locus(world_fmt ** universe, int usize, option_fmt * options,
                     estimateParameter(rep, *Gmax, EARTH, options,
                                       EARTH->cov[locus], chain, type, kind,
                                       EARTH->repkind);
-                print_progress(EARTH->options, EARTH, rep, steps * increment,
-                               accepted);
+                //print_progress(EARTH->options, EARTH, rep, steps * increment,
+                 //              accepted);
                 prepare_next_chain(universe, EARTH->options, type, chain,
                                    &chains, &pluschain, locus, replicate);
                 print_heating_progress(universe, EARTH->options, steps,
@@ -896,9 +902,9 @@ get_treedata(world_fmt * world, option_fmt * options)
 #ifdef SLOWNET
         else
         {
-            printf("%i> about to receive the sumfile data\n", myID);
+      //      printf("%i> about to receive the sumfile data\n", myID);
             mpi_results_master(MIGMPI_SUMFILE, world, maxreplicate, unpack_sumfile_buffer);
-            printf("%i> all sumfile data received and unpacked\n", myID);
+      //      printf("%i> all sumfile data received and unpacked\n", myID);
         }
 #endif
 
@@ -996,11 +1002,10 @@ print_progress(worldoption_fmt * options, world_fmt * world, long rep,
     if (options->progress)
     {
         print_menu_coalnodes(stdout, world, world->G + 1, rep);
-        print_menu_accratio(stdout, accepted, visited);
+        print_menu_accratio(accepted, visited, world);
         if (options->writelog)
         {
             print_menu_coalnodes(options->logfile, world, world->G + 1, rep);
-            print_menu_accratio(options->logfile, accepted, visited);
         }
         if (!world->start && world->chains > 1 && options->verbose)
             print_gelmanr(world->gelmanmeanR, world->gelmanmaxR);
@@ -1084,19 +1089,19 @@ print_heating_progress2(FILE * file, worldoption_fmt * options,
                         double acc_w, double acc_c, long hot, long warm,
                         long cold)
 {
-    fprintf(file, "           Swapping between %li temperatures.\n",
+    FPRINTF(file, "           Swapping between %li temperatures.\n",
             options->heated_chains);
     if (options->heated_chains > 4)
-        fprintf(file, "               Only the 4 coldest chains are shown\n");
-    fprintf(file,
+        FPRINTF(file, "               Only the 4 coldest chains are shown\n");
+    FPRINTF(file,
             "           Temperature | Accepted |  Swaps between temperatures\n");
-    fprintf(file, "           %10.4f  |   %4.2f   |          |\n",
+    FPRINTF(file, "           %10.4f  |   %4.2f   |          |\n",
             universe[3]->averageheat, acc_vh);
-    fprintf(file, "           %10.4f  |   %4.2f   |     %5li||\n",
+    FPRINTF(file, "           %10.4f  |   %4.2f   |     %5li||\n",
             universe[2]->averageheat, acc_h, hot);
-    fprintf(file, "           %10.4f  |   %4.2f   |     %5li ||\n",
+    FPRINTF(file, "           %10.4f  |   %4.2f   |     %5li ||\n",
             universe[1]->averageheat, acc_w, warm);
-    fprintf(file, "           %10.4f  |   %4.2f   |     %5li  |\n", universe[0]->averageheat,
+    FPRINTF(file, "           %10.4f  |   %4.2f   |     %5li  |\n", universe[0]->averageheat,
             acc_c, cold);
 }
 
@@ -1163,7 +1168,7 @@ analyze_olddata(world_fmt * world,
         print_options(world->outfile, world, options, data);
 #ifdef MPI
 
-        mpi_runloci_master(world->loci, world->who);
+        mpi_runloci_master(world->loci, world->who, world);
 #else
 
         for (locus = 0; locus < world->loci; locus++)
@@ -1238,14 +1243,14 @@ profile_tables(option_fmt * options, world_fmt * world, long *gmaxptr)
         for (i = 0; i < len; i++)
         {
             print_profile_likelihood_driver(i, world, gmaxptr);
-            fprintf(world->outfile, "%s\n\n", world->buffer);
+            FPRINTF(world->outfile, "%s\n\n", world->buffer);
             world->buffer[0] = '\0';
         }
 #endif
         if (myID == MASTER && options->printprofsummary)
         {
             print_profile_percentile(world);
-            fprintf(world->outfile, "%s\n\n", world->buffer);
+            FPRINTF(world->outfile, "%s\n\n", world->buffer);
             world->buffer[0] = '\0';
             destroy_profile_percentiles(world);
         }
@@ -1275,7 +1280,10 @@ setup_locus(long locus, world_fmt * world, option_fmt * options,
 {
     world->locus = locus;
     set_param(world, data, options, locus);
-    print_menu_locus(world, locus);
+	if(world->options->progress)
+		print_menu_locus(stdout,world, locus);
+	if(world->options->writelog)
+		print_menu_locus(world->options->logfile,world, locus);
     world->start = TRUE;
     buildtree(world, options, data, locus);
     if (world->replicate == 0)
