@@ -10,7 +10,7 @@ Copyright 2001 Peter Beerli and Joseph Felsenstein
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: migrate_mpi.c,v 1.18 2002/07/18 14:29:45 beerli Exp $
+$Id: migrate_mpi.c,v 1.22 2002/08/12 23:41:40 beerli Exp $
 */
 #ifdef MPI
 #include "migrate_mpi.h"
@@ -26,6 +26,7 @@ $Id: migrate_mpi.c,v 1.18 2002/07/18 14:29:45 beerli Exp $
 /*should go into profile.h*/
 #define GRIDSIZE 9
 
+
 extern void run_locus (world_fmt ** universe, int usize,
                            option_fmt * options, data_fmt * data,
                            tpool_t * heating_pool, long maxreplicate,
@@ -53,14 +54,24 @@ void swap_atl (long from, long to, world_fmt * world);
 long pack_quantile (char **buffer, quantile_fmt quant, long n);
 void unpack_quantile (char *buffer, quantile_fmt quant, long n);
 
+void handle_message(char *rawmessage,int sender, world_fmt *world);
+
+
+void set_filehandle(char *message, world_fmt *world,
+                    FILE **file, long *msgstart);
+
 void
-mpi_runloci_master (long loci, int *who)
+mpi_runloci_master (long loci, int *who, world_fmt *world)
 {
     long locus;
-    int sender;
-    long locusdone;
+    int sender = 0;
+    long locusdone = -1;
     MPI_Status status;
     long numsent = 0;
+    boolean done = FALSE;
+    char *rawmessage;
+    rawmessage = (char *) calloc(MAXBUFSIZE,sizeof(char));
+
     for (locus = 0; locus < MIN (loci, numcpu - 1); locus++)
     {
         MPI_Send (&locus, 1, MPI_LONG, locus + 1, locus + 1, comm_world);
@@ -68,9 +79,30 @@ mpi_runloci_master (long loci, int *who)
     }
     for (locus = 0; locus < loci; locus++)
     {
-        MPI_Recv (&locusdone, 1, MPI_LONG, MPI_ANY_SOURCE, MPI_ANY_TAG,
-                  comm_world, &status);
-        sender = status.MPI_SOURCE;
+        done=FALSE;
+        while(!done)
+        {
+            MPI_Recv (rawmessage, MAXBUFSIZE, MPI_CHAR, MPI_ANY_SOURCE, MPI_ANY_TAG,
+                      comm_world, &status);
+            sender = status.MPI_SOURCE;
+            //			fprintf(stdout,"%i> @@@@%s@@@@\n",sender, rawmessage);
+            switch(rawmessage[0])
+            {
+            case 'M':
+                sender = status.MPI_SOURCE;
+                handle_message(rawmessage,sender, world);
+                break;
+            case 'R':
+                //ignore first characeter and translate into locusnumber
+                locusdone = atol(rawmessage+1);
+                sender = status.MPI_SOURCE;
+                done=TRUE;
+                break;
+            default:
+                error("DIED because of wrong message from worker");
+                break;
+            }
+        }
         who[locusdone] = sender;
         if (numsent < loci)
         {
@@ -79,12 +111,14 @@ mpi_runloci_master (long loci, int *who)
         }
         else
         {
-            MPI_Send (&loci, 1, MPI_LONG, sender, 0, comm_world); //stop worker do wait for new loci
+            MPI_Send (&loci, 1, MPI_LONG, sender, 0, comm_world); //stop worker to wait for new loci
         }
     }
+    //stop worker that had never the chance to work on a locus, but are still
+    //listening
     for (sender = MIN (loci, numcpu - 1) + 1; sender < numcpu; sender++)
     {
-        MPI_Send (&loci, 1, MPI_LONG, sender, 0, comm_world); //stop worker do wait for new loci
+        MPI_Send (&loci, 1, MPI_LONG, sender, 0, comm_world); //stop worker to wait for new loci
     }
 }
 
@@ -97,7 +131,10 @@ mpi_runloci_worker (world_fmt ** universe, int usize,
 {
     boolean done = FALSE;
     long locus;
+    char *rawmessage;
+    long rawmsgsize = 0;
     MPI_Status status;
+    rawmessage = (char *) calloc(STRSIZE,sizeof(char));
     while (!done)
     {
         MPI_Recv (&locus, 1, MPI_LONG, MASTER, MPI_ANY_TAG,
@@ -106,7 +143,8 @@ mpi_runloci_worker (world_fmt ** universe, int usize,
         {
             run_locus (universe, usize, options, data,
                        heating_pool, maxreplicate, locus, treefilepos, Gmax);
-            MPI_Send (&locus, 1, MPI_LONG, MASTER, locus + 1, comm_world);
+            rawmsgsize = sprintf(rawmessage,"R%li",locus) + 1;
+            MPI_Send (rawmessage, rawmsgsize, MPI_CHAR, MASTER, locus + 1, comm_world);
             /* we want to know what locus we worked for
                - to control the work sent by master
                - to use in setup_parameter0() [combroyden2.c] */
@@ -506,36 +544,31 @@ pack_databuffer (char **buffer, data_fmt * data, option_fmt * options)
     char fp[LINESIZE];
     bufsize += LINESIZE;
     *buffer = (char *) realloc (*buffer, sizeof (char) * bufsize);
-    sprintf (fp, "%c %li %li %li\n", options->datatype, (long) data->hasghost,
-             data->numpop, data->loci);
-    bufsize += strlen (fp) + 1;
+    bufsize += 1 + sprintf (fp, "%c %li %li %li\n", options->datatype, (long) data->hasghost,
+                            data->numpop, data->loci);
     *buffer = (char *) realloc (*buffer, sizeof (char) * bufsize);
     strcat (*buffer, fp);
     for (locus = 0; locus < data->loci; locus++)
     {
-        sprintf (fp, "%li\n", data->seq->sites[locus]);
-        bufsize += strlen (fp) + 1;
+        bufsize += 1 + sprintf (fp, "%li\n", data->seq->sites[locus]);
         *buffer = (char *) realloc (*buffer, sizeof (char) * bufsize);
         strcat (*buffer, fp);
     }
-    sprintf (fp, "%li %f\n", data->seq->addon, data->seq->fracchange);
-    bufsize += strlen (fp) + 1;
+    bufsize += 1 + sprintf (fp, "%li %f\n", data->seq->addon, data->seq->fracchange);
     *buffer = (char *) realloc (*buffer, sizeof (char) * bufsize);
     strcat (*buffer, fp);
     // population data
     for (pop = 0; pop < data->numpop; pop++)
     {
-        sprintf (fp, "%s\n", data->popnames[pop]);
-        bufsize += strlen (fp) + 1;
+        bufsize += 1 + sprintf (fp, "%s\n", data->popnames[pop]);
         *buffer = (char *) realloc (*buffer, sizeof (char) * bufsize);
         strcat (*buffer, fp);
         biggest = 0;
         for(locus=0; locus<data->loci; locus++)
         {
-            sprintf (fp, "%li %li\n", data->numind[pop][locus],data->numalleles[pop][locus]);
+            bufsize += 1 + sprintf (fp, "%li %li\n", data->numind[pop][locus],data->numalleles[pop][locus]);
             if(biggest < data->numind[pop][locus])
                 biggest = data->numind[pop][locus];
-            bufsize += strlen (fp) + 1;
             *buffer = (char *) realloc (*buffer, sizeof (char) * bufsize);
             strcat (*buffer, fp);
         }
@@ -543,9 +576,8 @@ pack_databuffer (char **buffer, data_fmt * data, option_fmt * options)
         {
             for (ind = 0; ind < biggest; ind++)
             {
-                sprintf (fp, "%*.*s\n", (int) options->nmlength,
-                         (int) options->nmlength, data->indnames[pop][ind]);
-                bufsize += strlen (fp) + 1;
+                bufsize += 1 + sprintf (fp, "%*.*s\n", (int) options->nmlength,
+                                        (int) options->nmlength, data->indnames[pop][ind]);
                 *buffer = (char *) realloc (*buffer, sizeof (char) * bufsize);
                 strcat (*buffer, fp);
                 pack_allele_data (buffer, &bufsize, data, pop, ind);
@@ -557,9 +589,8 @@ pack_databuffer (char **buffer, data_fmt * data, option_fmt * options)
             {
                 for (ind = 0; ind < data->numind[pop][locus]; ind++)
                 {
-                    sprintf (fp, "%*.*s\n", (int) options->nmlength, (int) options->nmlength,
-                             data->indnames[pop][ind]);
-                    bufsize += strlen (fp) + 1;
+                    bufsize += 1 + sprintf (fp, "%*.*s\n", (int) options->nmlength, (int) options->nmlength,
+                                            data->indnames[pop][ind]);
                     *buffer = (char *) realloc (*buffer, sizeof (char) * bufsize);
                     strcat (*buffer, fp);
                     pack_sequence_data (buffer, &bufsize, data, pop, ind, locus);
@@ -570,10 +601,9 @@ pack_databuffer (char **buffer, data_fmt * data, option_fmt * options)
     // geofile
     if (options->geo)
     {
-        for (pop = 0; data->numpop * data->numpop; pop++)
+        for (pop = 0; pop < data->numpop * data->numpop; pop++)
         {
-            sprintf (fp, "%f %f\n", data->geo[pop], data->lgeo[pop]);
-            bufsize += strlen (fp) + 1;
+            bufsize += 1 + sprintf (fp, "%f %f\n", data->geo[pop], data->lgeo[pop]);
             *buffer = (char *) realloc (*buffer, sizeof (char) * bufsize);
             strcat (*buffer, fp);
         }
@@ -585,8 +615,7 @@ pack_databuffer (char **buffer, data_fmt * data, option_fmt * options)
         sumtips = 0;
         for (pop = 0; pop < data->numpop; ++pop)
             sumtips += data->numind[pop][0];//Assumes UEP is matched by locus 1
-        sprintf (fp, "%li %li\n", sumtips, data->uepsites);
-        bufsize += strlen (fp) + 1;
+        bufsize += 1 + sprintf (fp, "%li %li\n", sumtips, data->uepsites);
         *buffer = (char *) realloc (*buffer, sizeof (char) * bufsize);
         strcat (*buffer, fp);
         if (strchr (SEQUENCETYPES, options->datatype))
@@ -595,8 +624,7 @@ pack_databuffer (char **buffer, data_fmt * data, option_fmt * options)
             {
                 for (i = 0; i < data->uepsites; i++)
                 {
-                    sprintf (fp, "%i\n", data->uep[pop][i]);
-                    bufsize += strlen (fp) + 1;
+                    bufsize += 1 + sprintf (fp, "%i\n", data->uep[pop][i]);
                     *buffer =
                         (char *) realloc (*buffer, sizeof (char) * bufsize);
                     strcat (*buffer, fp);
@@ -609,9 +637,8 @@ pack_databuffer (char **buffer, data_fmt * data, option_fmt * options)
             {
                 for (i = 0; i < data->uepsites; i++)
                 {
-                    sprintf (fp, "%i %i\n", data->uep[pop][i],
-                             data->uep[pop + sumtips][i]);
-                    bufsize += strlen (fp) + 1;
+                    bufsize += 1 + sprintf (fp, "%i %i\n", data->uep[pop][i],
+                                            data->uep[pop + sumtips][i]);
                     *buffer =
                         (char *) realloc (*buffer, sizeof (char) * bufsize);
                     strcat (*buffer, fp);
@@ -632,9 +659,8 @@ pack_allele_data (char **buffer, long *bufsize, data_fmt * data, long pop,
     long locus;
     for (locus = 0; locus < data->loci; locus++)
     {
-        sprintf (fp, "%s %s\n", data->yy[pop][ind][locus][0],
-                 data->yy[pop][ind][locus][1]);
-        *bufsize += strlen (fp) + 1;
+        *bufsize += 1 + sprintf (fp, "%s %s\n", data->yy[pop][ind][locus][0],
+                                 data->yy[pop][ind][locus][1]);
         *buffer = (char *) realloc (*buffer, sizeof (char) * *bufsize);
         strcat (*buffer, fp);
     }
@@ -662,7 +688,7 @@ pack_sequence_data (char **buffer, long *bufsize, data_fmt * data, long pop,
 void
 unpack_databuffer (char *buffer, data_fmt * data, option_fmt * options)
 {
-    long locus, pop, ind, i;
+    long locus, pop, ind, i=0;
     long biggest;
 #ifdef UEP
 
@@ -740,7 +766,7 @@ unpack_databuffer (char *buffer, data_fmt * data, option_fmt * options)
     }
     else
     {
-        for (pop = 0; data->numpop * data->numpop; pop++)
+        for (pop = 0; pop < data->numpop * data->numpop; pop++)
         {
             sgets (input, LINESIZE, &buf);
             sscanf (input, "%lf%lf", &data->geo[pop], &data->lgeo[pop]);
@@ -864,14 +890,12 @@ pack_result_buffer (char **buffer, world_fmt * world,
 
     for (rep = 0; rep < maxrep + addon; rep++)
     {
-        sprintf (input, "%20.20f\n", atl[rep][locus].param_like);
-        bufsize += strlen (input) + 1;
+        bufsize += 1 + sprintf (input, "%20.20f\n", atl[rep][locus].param_like);
         (*buffer) = (char *) realloc (*buffer, sizeof (char) * bufsize);
         strcat (*buffer, input);
         for (pop = 0; pop < 4 * numpop * numpop; pop++)
         {
-            sprintf (input, "%20.20f\n", atl[rep][locus].parameters[pop]);
-            bufsize += strlen (input) + 1;
+            bufsize += 1 + sprintf (input, "%20.20f\n", atl[rep][locus].parameters[pop]);
             (*buffer) = (char *) realloc (*buffer, sizeof (char) * bufsize);
             strcat (*buffer, input);
         }
@@ -881,8 +905,7 @@ pack_result_buffer (char **buffer, world_fmt * world,
     {
         for (pop = 0; pop < world->options->lsteps; pop++)
         {
-            sprintf (input, "%20.20f\n", apg0[rep][locus][pop]);
-            bufsize += strlen (input) + 1;
+            bufsize += 1 + sprintf (input, "%20.20f\n", apg0[rep][locus][pop]);
             (*buffer) = (char *) realloc (*buffer, sizeof (char) * bufsize);
             strcat (*buffer, input);
         }
@@ -1264,8 +1287,8 @@ mpi_broadcast_results (world_fmt * world, long loci,
 #endif
 
     get_time (nowstr, "%H:%M:%S");
-	if(world->options->progress)
-    printf("%i> Redistributing the data\nResult parts [Time is %s]\n",myID, nowstr);
+    //if(world->options->progress)
+    // printf("%i> Redistributing the data\nResult parts [Time is %s]\n",myID, nowstr);
 #ifdef BIGMEMORY
 
     if (myID == MASTER)
@@ -1308,7 +1331,7 @@ mpi_broadcast_results (world_fmt * world, long loci,
             (*unpack)(allbuffer + offset, world, locus, maxreplicate,
                       world->numpop);
         }
-       // printf("%i> Loci results received\n",myID);
+        // printf("%i> Loci results received\n",myID);
     }
 #else
     for (locus = 0; locus < loci; locus++)
@@ -1319,7 +1342,7 @@ mpi_broadcast_results (world_fmt * world, long loci,
                               world->numpop);
             MPI_Bcast (&bufsize, 1, MPI_LONG, MASTER, comm_world);
             MPI_Bcast (allbuffer, bufsize, MPI_CHAR, MASTER, comm_world);
-           // printf("%i> Locus %li results sent\n",myID, locus);
+            // printf("%i> Locus %li results sent\n",myID, locus);
         }
         else
         {
@@ -1344,16 +1367,20 @@ void
 mpi_profiles_master (world_fmt * world, long nparam, int *profilewho)
 {
     long pnum;
-    int sender;
+    int sender=0;
     //  int i;
     long pdone;
+    boolean done;
     MPI_Status status;
-    long temp[3];
     long numsent = 0;
     long bufsize;
     long quantsize;
+    char *tempstr;
     char **buffer = &world->buffer;
     FILE *outfile = world->outfile;
+
+    tempstr = calloc(MAXBUFSIZE,sizeof(char));
+
     for (pnum = 0; pnum < MIN (nparam, numcpu - 1); pnum++)
     {
         MPI_Send (&pnum, 1, MPI_LONG, pnum + 1, pnum + 1, comm_world);
@@ -1361,25 +1388,41 @@ mpi_profiles_master (world_fmt * world, long nparam, int *profilewho)
     }
     for (pnum = 0; pnum < nparam; pnum++)
     {
-        MPI_Recv (temp, 3, MPI_LONG, MPI_ANY_SOURCE, MPI_ANY_TAG,
-                  comm_world, &status);
-        pdone = temp[0];
-        bufsize = temp[1];
-        quantsize = temp[2];
-        sender = status.MPI_SOURCE;
-        profilewho[pdone] = sender;
-        *buffer =
-            (char *) realloc (*buffer, sizeof (char) * (bufsize + quantsize + 1));
-        memset (*buffer, 0, sizeof (char) * (bufsize + quantsize + 1));
-        MPI_Recv (*buffer, bufsize + quantsize, MPI_CHAR, sender, MPI_ANY_TAG,
-                  comm_world, &status);
-        if(world->options->printprofsummary)
+        done = FALSE;
+        while(!done)
         {
-            unpack_quantile ((*buffer) + bufsize, world->quantiles[pdone],
-                             GRIDSIZE);
-            memset ((*buffer) + bufsize, 0, sizeof (char) * quantsize);
+            MPI_Recv (tempstr, MAXBUFSIZE, MPI_CHAR, MPI_ANY_SOURCE, MPI_ANY_TAG, comm_world, &status);
+            switch(tempstr[0])
+            {
+            case 'M':
+                sender = status.MPI_SOURCE;
+                handle_message(tempstr,sender, world);
+                break;
+            case 'P': sscanf(tempstr+1,"%li%li%li",&pdone,&bufsize,&quantsize);
+                //        pdone = temp[0];
+                //       bufsize = temp[1];
+                //      quantsize = temp[2];
+                sender = status.MPI_SOURCE;
+                profilewho[pdone] = sender;
+                *buffer =
+                    (char *) realloc (*buffer, sizeof (char) * (bufsize + quantsize + 1));
+                memset (*buffer, 0, sizeof (char) * (bufsize + quantsize + 1));
+                MPI_Recv (*buffer, bufsize + quantsize, MPI_CHAR, sender, MPI_ANY_TAG,
+                          comm_world, &status);
+                if(world->options->printprofsummary)
+                {
+                    unpack_quantile ((*buffer) + bufsize, world->quantiles[pdone],
+                                     GRIDSIZE);
+                    memset ((*buffer) + bufsize, 0, sizeof (char) * quantsize);
+                }
+                fprintf (outfile, "%s\n\n", *buffer);
+                done=TRUE;
+                break;
+            default:
+                error("DIED because of wrong message from worker");
+                break;
+            }
         }
-        fprintf (outfile, "%s\n\n", *buffer);
         if (numsent < nparam)
         {
             MPI_Send (&numsent, 1, MPI_LONG, sender, numsent + 1, comm_world);
@@ -1398,10 +1441,13 @@ mpi_profiles_worker (world_fmt * world, long *gmaxptr)
     boolean done = FALSE;
     long pnum;
     long temp[3];
+    char *tempstr;
+    long tempstrsize;
     char *quantilebuffer;
     MPI_Status status;
     long qbufsize = 1;
     quantilebuffer = (char *) calloc (qbufsize, sizeof (char));
+    tempstr = (char *) calloc (MAXBUFSIZE, sizeof (char));
     while (!done)
     {
         MPI_Recv (&pnum, 1, MPI_LONG, MASTER, MPI_ANY_TAG, comm_world, &status);
@@ -1420,7 +1466,8 @@ mpi_profiles_worker (world_fmt * world, long *gmaxptr)
             }
             else
                 temp[2] = 0;
-            MPI_Send (temp, 3, MPI_LONG, MASTER, pnum + 1, comm_world);
+            tempstrsize = 1 + sprintf(tempstr,"P%li %li %li", temp[0],temp[1],temp[2]);
+            MPI_Send (tempstr,tempstrsize, MPI_CHAR, MASTER, pnum + 1, comm_world);
             MPI_Send (world->buffer, temp[1] + temp[2], MPI_CHAR, MASTER,
                       pnum + 1, comm_world);
             world->profilewho[profiledone++] = pnum;
@@ -1430,6 +1477,7 @@ mpi_profiles_worker (world_fmt * world, long *gmaxptr)
             done = TRUE;
         }
     }
+    free(tempstr);
     free (quantilebuffer);
 }
 
@@ -1443,8 +1491,7 @@ pack_quantile (char **buffer, quantile_fmt quant, long n)
     sprintf (*buffer, "QUANTILEBUFFER:\n %s\n", quant.name);
     for (i = 0; i < n; i++)
     {
-        sprintf (fp, "%20.20f\n", quant.param[i]);
-        bufsize += strlen (fp) + 1;
+        bufsize += 1 + sprintf (fp, "%20.20f\n", quant.param[i]);
         *buffer = (char *) realloc (*buffer, sizeof (char) * bufsize);
         strcat (*buffer, fp);
     }
@@ -1660,4 +1707,137 @@ setup_parameter0_mpi (world_fmt * world, nr_fmt * nr, long repkind,
     }
 }
 
+
+void handle_message(char *rawmessage,int sender, world_fmt * world)
+{
+    char *rawptr;
+    long  pos=0;
+    FILE *file = stdout;
+    rawptr = rawmessage;
+    set_filehandle(rawmessage, world, &file, &pos);
+    fprintf(file,"%s", rawmessage + pos);
+    fflush(file);
+}
+
+// needs globals filedb, and filenum
+void setup_filehandle_db(FILE *file, world_fmt *world, option_fmt *options, data_fmt *data)
+{
+    long filehandle = get_filehandle(file, world, options, data);
+    filedb[filenum].file = file;
+    filedb[filenum++].handle = filehandle;
+    fprintf(stdout,"filedb: %p %li\n",file,filehandle);
+}
+
+long retrieve_filehandle(FILE *file)
+{
+    long i=0;
+    long filehandle = 0;
+    while(filedb[i].file != file && i<filenum)
+        i++;
+    if(i!=filenum)
+        filehandle = filedb[i].handle;
+    return filehandle;
+}
+
+long get_filehandle(FILE *file, world_fmt *world, option_fmt *options, data_fmt *data)
+{
+    if(file == stdout)
+        return STDOUTNUM;
+    if(file == options->logfile)
+        return LOGFILENUM;
+    if(file == world->outfile)
+        return OUTFILENUM;
+    if(file == options->aicfile)
+        return AICFILENUM;
+    if(file == world->mathfile)
+        return MATHFILENUM;
+#ifdef BAYESUPDATE	
+    if(file == world->bayesfile)
+        return BAYESFILENUM;
+#endif
+    return STDOUTNUM;
+}
+
+long get_filehandle2(FILE *file, world_fmt *world)
+{
+    if(file == stdout)
+        return STDOUTNUM;
+    if(file == world->options->logfile)
+        return LOGFILENUM;
+    if(file == world->outfile)
+        return OUTFILENUM;
+    if(file == world->mighistfile)
+        return MIGHISTFILENUM;
+    if(file == world->options->aicfile)
+        return AICFILENUM;
+    if(file == world->mathfile)
+        return MATHFILENUM;
+#ifdef BAYESUPDATE
+    if(file == world->bayesfile)
+        return BAYESFILENUM;
+#endif
+    return STDOUTNUM;
+}
+
+
+void set_filehandle(char *message, world_fmt *world,
+                    FILE **file, long *msgstart)
+{
+    long filepos=0;
+    static char *temp;
+    long filenum;
+    temp = calloc(100,sizeof(char));
+    filepos = strcspn(message,":") + 1;
+    *msgstart = filepos;
+    strncpy(temp,message,filepos);
+    filenum = atol(temp+1);
+    //fprintf(stdout,"\n@@@@@@@@@@@@@@@@@%li@%s@%li@\n",filenum,temp,filepos);
+    if(filenum==STDOUTNUM)
+    {
+        //		fprintf(stdout,"\n");
+        *file = stdout;
+        return;
+    }
+    if(filenum == LOGFILENUM)
+    {
+        //	fprintf(stdout," logfile\n");
+        *file = world->options->logfile;
+        return;
+    }
+    if(filenum == OUTFILENUM)
+    {
+        *file = world->outfile;
+        return ;
+    }
+    *file = stdout;
+    return;
+}
+
+void
+mpi_fprintf(FILE *file, const char *fmt, ...)
+{
+    char *p;
+    va_list ap;
+    long filehandle = 0;
+    long bufsize = 0;
+    p = calloc(MAXBUFSIZE,sizeof(char));
+    if(myID!=MASTER)
+    {
+        filehandle = retrieve_filehandle(file);
+        bufsize += sprintf(p, "%c%li:",'M',filehandle);
+    }
+    va_start(ap, fmt);
+    bufsize += 1+ vsprintf(p+bufsize, fmt, ap);
+    if(myID!=MASTER)
+    {
+        bufsize= 1+strlen(p);
+        MPI_Send (p, bufsize, MPI_CHAR, MASTER, myID, comm_world);
+    }
+    else
+        fprintf(file,"%s", p);
+    va_end(ap);
+   // fprintf(stderr,"%s\n", p);
+    free(p);
+}
+
 #endif
