@@ -3,6 +3,18 @@
 #ifdef MPI
 #include <mpi.h>
 #include "migration.h"
+
+#include <stdarg.h>
+
+typedef struct _filedb_fmt {
+	FILE *file;
+	long handle;
+} filedb_fmt;
+
+
+extern filedb_fmt filedb[20];
+extern long filenum;
+
 extern MPI_Comm comm_world;
 //extern MPI_Comm comm_all_worlds;
 #endif
@@ -33,7 +45,7 @@ extern int profiledone;
 
 extern void broadcast_options (option_fmt * options);
 
-extern void mpi_runloci_master (long loci, int *who);
+extern void mpi_runloci_master (long loci, int *who, world_fmt *world);
 
 extern void mpi_runloci_worker (world_fmt ** universe, int usize,
                                     option_fmt * options, data_fmt * data,
@@ -85,6 +97,11 @@ extern void assignloci_worker (world_fmt * world);
 void setup_parameter0_mpi (world_fmt * world, nr_fmt * nr, long repkind,
                            long repstart, long repstop, long loci, long kind,
                            boolean multilocus);
+
+extern void setup_filehandle_db(FILE *file, world_fmt *world, option_fmt *options, data_fmt *data);
+extern long get_filehandle(FILE *file, world_fmt *world, option_fmt *options, data_fmt *data);
+extern void mpi_fprintf(FILE *file, const char *fmt, ...);
+extern long get_filehandle2(FILE *file, world_fmt *world);
 #ifdef SLOWNET
 void
 mpi_broadcast_results (world_fmt * world, long loci,
@@ -98,5 +115,7 @@ void setup_parameter0_slowmpi (world_fmt * world, nr_fmt * nr, long repkind,
                                long repstart, long repstop, long loci,
                                long kind, boolean multilocus);
 #endif /*slownet */
+
+
 #endif /*mpi */
 #endif
