@@ -19,7 +19,7 @@ Copyright 2001 Peter Beerli and Joseph Felsenstein
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: tools.c,v 1.55 2002/07/12 18:14:08 beerli Exp $
+$Id: tools.c,v 1.56 2002/08/12 21:13:57 beerli Exp $
 -------------------------------------------------------*/
 #include "migration.h"
 #include "data.h"
@@ -204,6 +204,13 @@ logfac (long n)
 void
 init_files (world_fmt * world, data_fmt * data, option_fmt * options)
 {
+	long *tempdb;
+#ifdef MPI
+	char tempfilenum=0;
+	long i,j;
+#endif
+	tempdb = calloc(20*2+1,sizeof(long));
+	
     if (myID == MASTER)
     {
 		if (!options->readsum)
@@ -224,14 +231,32 @@ init_files (world_fmt * world, data_fmt * data, option_fmt * options)
 		}
 		else
 			openfile (&data->sumfile, options->sumfilename, "r+", appl, NULL);
-		
+
 		openfile (&world->outfile, options->outfilename, "w+", appl, NULL);
+#ifdef MPI
+		setup_filehandle_db((void *) world->outfile, world, options,data);
+#endif
 		if (options->mighist)
+		  {
 			openfile (&world->mighistfile, options->mighistfilename, "w+", appl,NULL);
+#ifdef MPI
+			setup_filehandle_db((void *) world->mighistfile, world, options, data);
+#endif			
+		  }
 		if (options->writelog)
+		  {
 			openfile (&options->logfile, options->logfilename, "w+", appl, NULL);
+#ifdef MPI
+			setup_filehandle_db((void *) options->logfile, world, options, data);
+#endif			
+		  }
 		if (options->aic)
+		  {
 			openfile (&options->aicfile, options->aicfilename, "w+", appl, NULL);
+#ifdef MPI
+			setup_filehandle_db((void *) options->aicfile, world, options, data);
+#endif
+		  }			
         if (options->plot)
         {
             switch (options->plotmethod)
@@ -239,25 +264,71 @@ init_files (world_fmt * world, data_fmt * data, option_fmt * options)
             case PLOTALL:
                 openfile (&world->mathfile, options->mathfilename, "w+", appl,
                           NULL);
+#ifdef MPI
+				setup_filehandle_db((void *) world->mathfile, world, options, data);
+#endif
                 break;
             default:  /*e.g. 0 this create just the plots in outfile */
                 break;
             }
         }
         if (options->geo)
+		  {
             openfile (&data->geofile, options->geofilename, "r+", appl, NULL);
+		  }
 #ifdef UEP
 
         if (options->uep)
             openfile (&data->uepfile, options->uepfilename, "r+", appl, NULL);
 #endif
 #ifdef BAYESUPDATE
-
         if (options->bayes_infer)
+		  {
             openfile (&world->bayesfile, options->bayesfilename, "w+", appl, NULL);
+#ifdef MPI
+				setup_filehandle_db((void *) world->bayesfile, world);
+#endif
+		  }			
+#endif /*BAYESUPDATE*/
+#ifdef MPI
+		tempdb[0]=filenum;
+		for(i=0, j=1;i<filenum;i++)
+		{
+			tempdb[j++] = (long) filedb[i].file;
+			tempdb[j++] = (long) filedb[i].handle;
+		}
+		tempfilenum = filenum * 2 + 1;
+		MPI_Bcast (tempdb, tempfilenum, MPI_LONG, MASTER, comm_world);
 #endif
-
     }
+	else
+	  {
+		// all non myID==MASTER
+#ifdef MPI
+		MPI_Bcast (tempdb, 20*2+1, MPI_LONG, MASTER, comm_world);
+		filenum = tempdb[0]; 
+		for(i=0, j=1;i<filenum;i++)
+		  {
+			filedb[i].file = (FILE *) tempdb[j++];
+			filedb[i].file = (FILE *) tempdb[j++];
+		  }
+		//assumess same order as in the master
+		j=0;
+		world->outfile = filedb[j++].file;
+		if (options->mighist)
+			world->mighistfile = filedb[j++].file;
+		if (options->writelog)
+			options->logfile = filedb[j++].file;
+		if (options->aic)
+			options->aicfile = filedb[j++].file;
+		if (options->plot && options->plotmethod == PLOTALL)
+			world->mathfile = filedb[j++].file;
+#ifdef BAYESUPDATE
+        if (options->bayes_infer)
+			world->bayesfile = filedb[j++].file;
+#endif /*bayesupdate*/
+#endif /*MPI*/
+	  }
 }
 
 void
@@ -481,7 +552,6 @@ openfile (FILE ** fp, char *filename, char *mode, const char *application,
     if (perm != NULL)
         strcpy (perm, file);
     strcpy (filename, file);
-
 }
 
 long
@@ -665,7 +735,7 @@ write_savesum (world_fmt * world)
                 for (j = 0; j < world->numpop2; j++)
                 {
                     fprintf (sumfile, "%f ", ta[r][l].tl[i].mindex[j]);
-                    //debug               fprintf (sumfile, "%li ", ta[l].tl[i].l[j]);
+                    //debug               printf (sumfile, "%li ", ta[l].tl[i].l[j]);
                 }
                 fprintf (sumfile, "\n");
             }
@@ -676,9 +746,9 @@ write_savesum (world_fmt * world)
             }
             //  for(i=0;i<ta[l].T;i++)
             //        {
-            //          fprintf(sumfile,"%20.20e ",ta[l].likelihood[i]);
+            //          printf(sumfile,"%20.20e ",ta[l].likelihood[i]);
             //        }
-            //      fprintf(sumfile,"\n");
+            //      printf(sumfile,"\n");
             fprintf (sumfile, "%li %20.20e\n", ta[r][l].trials, ta[r][l].normd);
         }
     }
@@ -1190,7 +1260,7 @@ lu_decomp (double **m, long *indeks, long nrows)
                 summ -= m[k][p] * m[p][i];
             m[k][i] = summ;
             pivot = fabs (summ) /**max_row_vals[k]*/ ;
-            /*  fprintf(stdout,"i=%li,pivot=%f,big=%f\n",i,pivot,big); */
+            /*  printf(stdout,"i=%li,pivot=%f,big=%f\n",i,pivot,big); */
             if (pivot >= big)
             {
                 big = pivot;
@@ -2320,7 +2390,7 @@ calc_gamma (double alpha, double *gama, long categs)
         }
         gama[i] = x / alpha;
         //Debug
-        //      fprintf (stderr, "  %li> %f\n", i, gama[i]);
+        //      printf (stderr, "  %li> %f\n", i, gama[i]);
 
         if (x >= 10e10)
         {
